"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.debugMarkerToString = exports.internedSpaces = exports.compareStrings = exports.compareNumbers = exports.CollaborationWindow = exports.IncrementalMapState = exports.IncrementalExecOp = exports.Marker = exports.reservedMarkerSimpleTypeKey = exports.reservedMarkerIdKey = exports.BaseSegment = exports.seqLTE = exports.MergeBlock = exports.MaxNodesInBlock = exports.MergeNode = exports.toRemovalInfo = void 0;
/* eslint-disable @typescript-eslint/no-non-null-assertion */
/* eslint-disable import/no-deprecated */
const core_utils_1 = require("@fluidframework/core-utils");
const constants_1 = require("./constants");
const localReference_1 = require("./localReference");
const mergeTreeTracking_1 = require("./mergeTreeTracking");
const ops_1 = require("./ops");
const ordinal_1 = require("./ordinal");
const properties_1 = require("./properties");
const referencePositions_1 = require("./referencePositions");
const segmentGroupCollection_1 = require("./segmentGroupCollection");
const segmentPropertiesManager_1 = require("./segmentPropertiesManager");
/**
 * @deprecated This functionality was not meant to be exported and will be removed in a future release
 * @internal
 */
function toRemovalInfo(maybe) {
    if (maybe?.removedClientIds !== undefined && maybe?.removedSeq !== undefined) {
        return maybe;
    }
    (0, core_utils_1.assert)(maybe?.removedClientIds === undefined && maybe?.removedSeq === undefined, 0x2bf /* "both removedClientIds and removedSeq should be set or not set" */);
}
exports.toRemovalInfo = toRemovalInfo;
/**
 * @internal
 */
class MergeNode {
    constructor() {
        this.index = 0;
        this.ordinal = "";
        this.cachedLength = 0;
    }
    isLeaf() {
        return false;
    }
}
exports.MergeNode = MergeNode;
/**
 * Note that the actual branching factor of the MergeTree is `MaxNodesInBlock - 1`.  This is because
 * the MergeTree always inserts first, then checks for overflow and splits if the child count equals
 * `MaxNodesInBlock`.  (i.e., `MaxNodesInBlock` contains 1 extra slot for temporary storage to
 * facilitate splits.)
 * @internal
 */
exports.MaxNodesInBlock = 8;
/**
 * @internal
 */
class MergeBlock extends MergeNode {
    constructor(childCount) {
        super();
        this.childCount = childCount;
        this.children = new Array(exports.MaxNodesInBlock);
    }
    hierBlock() {
        return undefined;
    }
    setOrdinal(child, index) {
        const childCount = this.childCount;
        (0, core_utils_1.assert)(childCount >= 1 && childCount <= exports.MaxNodesInBlock, 0x040 /* "Child count is not within [1,8] range!" */);
        child.ordinal = (0, ordinal_1.computeHierarchicalOrdinal)(exports.MaxNodesInBlock, childCount, this.ordinal, index === 0 ? undefined : this.children[index - 1]?.ordinal);
    }
    assignChild(child, index, updateOrdinal = true) {
        child.parent = this;
        child.index = index;
        if (updateOrdinal) {
            this.setOrdinal(child, index);
        }
        this.children[index] = child;
    }
}
exports.MergeBlock = MergeBlock;
function seqLTE(seq, minOrRefSeq) {
    return seq !== constants_1.UnassignedSequenceNumber && seq <= minOrRefSeq;
}
exports.seqLTE = seqLTE;
/**
 * @internal
 */
class BaseSegment extends MergeNode {
    constructor() {
        super(...arguments);
        this.clientId = constants_1.LocalClientId;
        this.seq = constants_1.UniversalSequenceNumber;
        this.segmentGroups = new segmentGroupCollection_1.SegmentGroupCollection(this);
        this.trackingCollection = new mergeTreeTracking_1.TrackingGroupCollection(this);
    }
    addProperties(newProps, op, seq, collabWindow, rollback = segmentPropertiesManager_1.PropertiesRollback.None) {
        this.propertyManager ?? (this.propertyManager = new segmentPropertiesManager_1.PropertiesManager());
        this.properties ?? (this.properties = (0, properties_1.createMap)());
        return this.propertyManager.addProperties(this.properties, newProps, op, seq, collabWindow && collabWindow.collaborating, rollback);
    }
    hasProperty(key) {
        return !!this.properties && this.properties[key] !== undefined;
    }
    isLeaf() {
        return true;
    }
    cloneInto(b) {
        b.clientId = this.clientId;
        // TODO: deep clone properties
        b.properties = (0, properties_1.clone)(this.properties);
        b.removedClientIds = this.removedClientIds?.slice();
        // TODO: copy removed client overlap and branch removal info
        b.removedSeq = this.removedSeq;
        b.seq = this.seq;
        b.attribution = this.attribution?.clone();
    }
    canAppend(segment) {
        return false;
    }
    addSerializedProps(jseg) {
        if (this.properties) {
            jseg.props = this.properties;
        }
    }
    /**
     * @deprecated This functionality was not meant to be exported and will be removed in a future release
     */
    ack(segmentGroup, opArgs) {
        const currentSegmentGroup = this.segmentGroups.dequeue();
        (0, core_utils_1.assert)(currentSegmentGroup === segmentGroup, 0x043 /* "On ack, unexpected segmentGroup!" */);
        switch (opArgs.op.type) {
            case ops_1.MergeTreeDeltaType.ANNOTATE:
                (0, core_utils_1.assert)(!!this.propertyManager, 0x044 /* "On annotate ack, missing segment property manager!" */);
                this.propertyManager.ackPendingProperties(opArgs.op);
                return true;
            case ops_1.MergeTreeDeltaType.INSERT:
                (0, core_utils_1.assert)(this.seq === constants_1.UnassignedSequenceNumber, 0x045 /* "On insert, seq number already assigned!" */);
                this.seq = opArgs.sequencedMessage.sequenceNumber;
                this.localSeq = undefined;
                return true;
            case ops_1.MergeTreeDeltaType.REMOVE:
                const removalInfo = toRemovalInfo(this);
                (0, core_utils_1.assert)(removalInfo !== undefined, 0x046 /* "On remove ack, missing removal info!" */);
                this.localRemovedSeq = undefined;
                if (removalInfo.removedSeq === constants_1.UnassignedSequenceNumber) {
                    removalInfo.removedSeq = opArgs.sequencedMessage.sequenceNumber;
                    return true;
                }
                return false;
            default:
                throw new Error(`${opArgs.op.type} is in unrecognized operation type`);
        }
    }
    splitAt(pos) {
        if (pos > 0) {
            const leafSegment = this.createSplitSegmentAt(pos);
            if (leafSegment) {
                this.copyPropertiesTo(leafSegment);
                // eslint-disable-next-line @typescript-eslint/no-this-alias
                const thisAsMergeSegment = this;
                leafSegment.parent = thisAsMergeSegment.parent;
                // Give the leaf a temporary yet valid ordinal.
                // when this segment is put in the tree, it will get its real ordinal,
                // but this ordinal meets all the necessary invariants for now.
                leafSegment.ordinal = this.ordinal + String.fromCharCode(0);
                leafSegment.removedClientIds = this.removedClientIds?.slice();
                leafSegment.removedSeq = this.removedSeq;
                leafSegment.localRemovedSeq = this.localRemovedSeq;
                leafSegment.seq = this.seq;
                leafSegment.localSeq = this.localSeq;
                leafSegment.clientId = this.clientId;
                this.segmentGroups.copyTo(leafSegment);
                this.trackingCollection.copyTo(leafSegment);
                if (this.localRefs) {
                    this.localRefs.split(pos, leafSegment);
                }
                if (this.attribution) {
                    leafSegment.attribution = this.attribution.splitAt(pos);
                }
            }
            return leafSegment;
        }
    }
    copyPropertiesTo(other) {
        if (this.propertyManager) {
            if (this.properties) {
                other.propertyManager = new segmentPropertiesManager_1.PropertiesManager();
                other.properties = this.propertyManager.copyTo(this.properties, other.properties, other.propertyManager);
            }
        }
    }
    append(other) {
        // Note: Must call 'appendLocalRefs' before modifying this segment's length as
        //       'this.cachedLength' is used to adjust the offsets of the local refs.
        localReference_1.LocalReferenceCollection.append(this, other);
        if (this.attribution) {
            (0, core_utils_1.assert)(other.attribution !== undefined, 0x4bd /* attribution should be set on appendee */);
            this.attribution.append(other.attribution);
        }
        else {
            (0, core_utils_1.assert)(other.attribution === undefined, 0x4be /* attribution should not be set on appendee */);
        }
        this.cachedLength ?? (this.cachedLength = 0);
        this.cachedLength += other.cachedLength;
    }
}
exports.BaseSegment = BaseSegment;
/**
 * @internal
 */
exports.reservedMarkerIdKey = "markerId";
/**
 * @internal
 */
exports.reservedMarkerSimpleTypeKey = "markerSimpleType";
/**
 * @internal
 */
class Marker extends BaseSegment {
    static is(segment) {
        return segment.type === Marker.type;
    }
    static make(refType, props) {
        const marker = new Marker(refType);
        if (props) {
            marker.addProperties(props);
        }
        return marker;
    }
    constructor(refType) {
        super();
        this.refType = refType;
        this.type = Marker.type;
        this.cachedLength = 1;
    }
    toJSONObject() {
        const obj = { marker: { refType: this.refType } };
        super.addSerializedProps(obj);
        return obj;
    }
    static fromJSONObject(spec) {
        if (spec && typeof spec === "object" && "marker" in spec) {
            return Marker.make(spec.marker.refType, spec.props);
        }
        return undefined;
    }
    clone() {
        const b = Marker.make(this.refType, this.properties);
        this.cloneInto(b);
        return b;
    }
    getSegment() {
        return this;
    }
    getOffset() {
        return 0;
    }
    hasSimpleType(simpleTypeName) {
        return !!this.properties && this.properties[exports.reservedMarkerSimpleTypeKey] === simpleTypeName;
    }
    getProperties() {
        return this.properties;
    }
    getId() {
        return this.properties?.[exports.reservedMarkerIdKey];
    }
    toString() {
        return `M${this.getId()}`;
    }
    createSplitSegmentAt(pos) {
        return undefined;
    }
    canAppend(segment) {
        return false;
    }
    append() {
        throw new Error("Can not append to marker");
    }
}
exports.Marker = Marker;
Marker.type = "Marker";
/**
 * @internal
 */
var IncrementalExecOp;
(function (IncrementalExecOp) {
    IncrementalExecOp[IncrementalExecOp["Go"] = 0] = "Go";
    IncrementalExecOp[IncrementalExecOp["Stop"] = 1] = "Stop";
    IncrementalExecOp[IncrementalExecOp["Yield"] = 2] = "Yield";
})(IncrementalExecOp || (exports.IncrementalExecOp = IncrementalExecOp = {}));
/**
 * @internal
 */
class IncrementalMapState {
    constructor(block, actions, pos, refSeq, clientId, context, start, end, childIndex = 0) {
        this.block = block;
        this.actions = actions;
        this.pos = pos;
        this.refSeq = refSeq;
        this.clientId = clientId;
        this.context = context;
        this.start = start;
        this.end = end;
        this.childIndex = childIndex;
        this.op = IncrementalExecOp.Go;
    }
}
exports.IncrementalMapState = IncrementalMapState;
/**
 * @deprecated This functionality was not meant to be exported and will be removed in a future release
 * @internal
 */
class CollaborationWindow {
    constructor() {
        this.clientId = constants_1.LocalClientId;
        this.collaborating = false;
        // Lowest-numbered segment in window; no client can reference a state before this one
        this.minSeq = 0;
        // Highest-numbered segment in window and current
        // reference segment for this client
        this.currentSeq = 0;
        this.localSeq = 0;
    }
    loadFrom(a) {
        this.clientId = a.clientId;
        this.collaborating = a.collaborating;
        this.minSeq = a.minSeq;
        this.currentSeq = a.currentSeq;
    }
}
exports.CollaborationWindow = CollaborationWindow;
/**
 * @deprecated This functionality was not meant to be exported and will be removed in a future release
 * @internal
 */
const compareNumbers = (a, b) => a - b;
exports.compareNumbers = compareNumbers;
/**
 * @deprecated This functionality was not meant to be exported and will be removed in a future release
 * @internal
 */
const compareStrings = (a, b) => a.localeCompare(b);
exports.compareStrings = compareStrings;
const indentStrings = ["", " ", "  "];
/**
 * @deprecated This functionality is deprecated and will be removed in a future release.
 * @internal
 */
function internedSpaces(n) {
    if (indentStrings[n] === undefined) {
        indentStrings[n] = "";
        for (let i = 0; i < n; i++) {
            indentStrings[n] += " ";
        }
    }
    return indentStrings[n];
}
exports.internedSpaces = internedSpaces;
/**
 * @internal
 */
function debugMarkerToString(marker) {
    let bbuf = "";
    if ((0, referencePositions_1.refTypeIncludesFlag)(marker, ops_1.ReferenceType.Tile)) {
        bbuf += "Tile";
    }
    if ((0, referencePositions_1.refTypeIncludesFlag)(marker, ops_1.ReferenceType.NestBegin)) {
        if (bbuf.length > 0) {
            bbuf += "; ";
        }
        bbuf += "RangeBegin";
    }
    if ((0, referencePositions_1.refTypeIncludesFlag)(marker, ops_1.ReferenceType.NestEnd)) {
        if (bbuf.length > 0) {
            bbuf += "; ";
        }
        bbuf += "RangeEnd";
    }
    let lbuf = "";
    const id = marker.getId();
    if (id) {
        bbuf += ` (${id}) `;
    }
    const tileLabels = (0, referencePositions_1.refGetTileLabels)(marker);
    if (tileLabels) {
        lbuf += "tile -- ";
        for (let i = 0, len = tileLabels.length; i < len; i++) {
            const tileLabel = tileLabels[i];
            if (i > 0) {
                lbuf += "; ";
            }
            lbuf += tileLabel;
        }
    }
    const rangeLabels = (0, referencePositions_1.refGetRangeLabels)(marker);
    if (rangeLabels) {
        let rangeKind = "begin";
        if ((0, referencePositions_1.refTypeIncludesFlag)(marker, ops_1.ReferenceType.NestEnd)) {
            rangeKind = "end";
        }
        if (tileLabels) {
            lbuf += " ";
        }
        lbuf += `range ${rangeKind} -- `;
        const labels = rangeLabels;
        for (let i = 0, len = labels.length; i < len; i++) {
            const rangeLabel = labels[i];
            if (i > 0) {
                lbuf += "; ";
            }
            lbuf += rangeLabel;
        }
    }
    let pbuf = "";
    if (marker.properties) {
        pbuf += JSON.stringify(marker.properties, (key, value) => {
            // Avoid circular reference when stringifying makers containing handles.
            // (Substitute a debug string instead.)
            const handle = !!value && value.IFluidHandle;
            // eslint-disable-next-line @typescript-eslint/no-unsafe-return
            return handle ? `#Handle(${handle.routeContext.path}/${handle.path})` : value;
        });
    }
    return `M ${bbuf}: ${lbuf} ${pbuf}`;
}
exports.debugMarkerToString = debugMarkerToString;
//# sourceMappingURL=mergeTreeNodes.js.map