/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
/**
 * Flags enum that dictates behavior of a ReferencePosition
 * @internal
 */
export var ReferenceType;
(function (ReferenceType) {
    ReferenceType[ReferenceType["Simple"] = 0] = "Simple";
    /**
     * Allows this reference to be located using the `findTile` API on merge-tree.
     */
    ReferenceType[ReferenceType["Tile"] = 1] = "Tile";
    /**
     * @deprecated this functionality is no longer supported and will be removed
     */
    ReferenceType[ReferenceType["NestBegin"] = 2] = "NestBegin";
    /**
     * @deprecated this functionality is no longer supported and will be removed
     */
    ReferenceType[ReferenceType["NestEnd"] = 4] = "NestEnd";
    ReferenceType[ReferenceType["RangeBegin"] = 16] = "RangeBegin";
    ReferenceType[ReferenceType["RangeEnd"] = 32] = "RangeEnd";
    /**
     * When a segment is marked removed (locally or with ack), this reference will slide to the first
     * valid option of:
     * 1. the start of the next furthest segment
     * 2. the end of the next nearest segment
     * 3. DetachedReferencePosition
     */
    ReferenceType[ReferenceType["SlideOnRemove"] = 64] = "SlideOnRemove";
    /**
     * When a segment is marked removed (locally or with ack), this reference will remain on that segment.
     */
    ReferenceType[ReferenceType["StayOnRemove"] = 128] = "StayOnRemove";
    /**
     * Specifies that the reference position should never be added to the segment it refers to.
     * This is useful for comparison/iteration purposes
     */
    ReferenceType[ReferenceType["Transient"] = 256] = "Transient";
})(ReferenceType || (ReferenceType = {}));
// Note: Assigned positive integers to avoid clashing with MergeTreeMaintenanceType
/**
 * @internal
 */
export const MergeTreeDeltaType = {
    INSERT: 0,
    REMOVE: 1,
    ANNOTATE: 2,
    /**
     * @deprecated The ability to create group ops will be removed in an upcoming release, as group ops are redundant with he native batching capabilities of the runtime
     */
    GROUP: 3,
};
//# sourceMappingURL=ops.js.map